package en.MaxedOut.UltraRespawn.Events;

import org.bukkit.event.player.PlayerQuitEvent;
import org.bukkit.SoundCategory;
import org.bukkit.Statistic;
import org.bukkit.entity.ExperienceOrb;
import org.bukkit.plugin.PluginManager;
import org.bukkit.inventory.PlayerInventory;
import org.bukkit.entity.Entity;
import org.bukkit.event.Event;
import org.bukkit.Bukkit;
import en.MaxedOut.UltraRespawn.Utils.XpHandler;
import org.bukkit.inventory.ItemStack;
import org.bukkit.Sound;
import org.bukkit.event.entity.EntityDamageByEntityEvent;
import org.bukkit.event.entity.EntityDamageEvent;
import org.bukkit.entity.Damageable;
import org.bukkit.util.Vector;
import net.citizensnpcs.api.npc.NPC;
import org.bukkit.metadata.MetadataValue;
import org.bukkit.metadata.FixedMetadataValue;
import org.bukkit.entity.LivingEntity;
import org.bukkit.event.player.PlayerTeleportEvent;
import org.bukkit.GameRule;
import org.bukkit.entity.EntityType;
import org.bukkit.Location;
import org.bukkit.potion.PotionEffect;
import org.bukkit.potion.PotionEffectType;
import org.bukkit.plugin.Plugin;
import en.MaxedOut.UltraRespawn.Main;
import java.util.Iterator;
import org.bukkit.World;
import org.bukkit.scheduler.BukkitRunnable;
import org.bukkit.event.player.PlayerRespawnEvent;
import org.bukkit.event.EventPriority;
import org.bukkit.event.EventHandler;
import org.bukkit.Material;
import org.bukkit.event.entity.PlayerDeathEvent;
import org.bukkit.event.Cancellable;
import java.util.ArrayList;
import net.citizensnpcs.api.npc.NPCDataStore;
import net.citizensnpcs.api.CitizensAPI;
import net.citizensnpcs.api.npc.MemoryNPCDataStore;
import org.bukkit.entity.Player;
import java.util.List;
import en.MaxedOut.UltraRespawn.Utils.DeathMessageHandler;
import en.MaxedOut.UltraRespawn.Config;
import net.citizensnpcs.api.npc.NPCRegistry;
import org.bukkit.event.Listener;

public class PlayerDeath implements Listener
{
    private final NPCRegistry registry;
    private Config config;
    private DeathMessageHandler dmh;
    private boolean cancellable;
    private List<Player> cantDamage;
    
    public PlayerDeath(final Config config, final DeathMessageHandler dmh) {
        this.registry = CitizensAPI.createAnonymousNPCRegistry((NPCDataStore)new MemoryNPCDataStore());
        this.cancellable = true;
        this.cantDamage = new ArrayList<Player>();
        this.config = config;
        this.dmh = dmh;
        if (!Cancellable.class.isAssignableFrom(PlayerDeathEvent.class)) {
            this.cancellable = false;
        }
    }
    
    @EventHandler(ignoreCancelled = true, priority = EventPriority.HIGHEST)
    public void onPlayerDeath(final PlayerDeathEvent e) {
        if (!this.config.enabled) {
            return;
        }
        final Player p = e.getEntity();
        if (p.hasMetadata("DeathAnimation")) {
            e.setDeathMessage("");
        }
        if (p.hasMetadata("NPC")) {
            return;
        }
        if (this.config.alternateMethod && this.cancellable) {
            e.setCancelled(true);
            this.fakeRespawn(p, e);
            return;
        }
        if (this.config.alternateMethod) {
            return;
        }
        if (this.config.tnttagDisable && p.getInventory().getHelmet() != null && p.getInventory().getHelmet().getType() == Material.TNT) {
            return;
        }
        this.npcCreate(p, true);
    }
    
    @EventHandler
    public void onPlayerRespawn(final PlayerRespawnEvent e) {
        if (!this.config.enabled) {
            return;
        }
        final Player p = e.getPlayer();
        if (p.hasMetadata("NPC")) {
            return;
        }
        if (!this.config.enabled) {
            return;
        }
        final World w = p.getWorld();
        if (this.config.spawnProtection) {
            this.cantDamage.add(p);
            new BukkitRunnable() {
                public void run() {
                    if (PlayerDeath.this.config.spawnProtectFlash) {
                        final List<Player> players = w.getPlayers();
                        new BukkitRunnable() {
                            boolean hidden = false;
                            
                            public void run() {
                                if (!PlayerDeath.this.cantDamage.contains(p) || !p.isOnline()) {
                                    for (final Player player : players) {
                                        if (player.isOnline()) {
                                            player.showPlayer(p);
                                            this.hidden = false;
                                        }
                                    }
                                    this.cancel();
                                    return;
                                }
                                if (!this.hidden) {
                                    for (final Player player : players) {
                                        if (player.isOnline()) {
                                            player.hidePlayer(p);
                                            this.hidden = true;
                                        }
                                    }
                                }
                                else {
                                    for (final Player player : players) {
                                        if (player.isOnline()) {
                                            player.showPlayer(p);
                                            this.hidden = false;
                                        }
                                    }
                                }
                            }
                        }.runTaskTimer((Plugin)Main.instance, 0L, 3L);
                    }
                    new BukkitRunnable() {
                        public void run() {
                            if (PlayerDeath.this.cantDamage.contains(p)) {
                                PlayerDeath.this.cantDamage.remove(p);
                            }
                        }
                    }.runTaskLater((Plugin)Main.instance, (long)PlayerDeath.this.config.spawnProtectionTime);
                }
            }.runTaskLater((Plugin)Main.instance, 1L);
        }
        new BukkitRunnable() {
            public void run() {
                if (PlayerDeath.this.config.blindness) {
                    final PotionEffect effect = new PotionEffect(PotionEffectType.BLINDNESS, PlayerDeath.this.config.blindnessTime, 0, false, false, false);
                    p.addPotionEffect(effect);
                }
            }
        }.runTask((Plugin)Main.instance);
        if (this.config.playSound) {
            new BukkitRunnable() {
                public void run() {
                    p.playSound(p.getLocation(), PlayerDeath.this.config.sound, 1.0f, 1.0f);
                    if (PlayerDeath.this.config.spawnParticles) {
                        final Location l = p.getLocation();
                        p.getWorld().spawnParticle(PlayerDeath.this.config.spawnParticleType, l.getX(), l.getY() + 0.7, l.getZ(), 50, 0.1, 0.1, 0.1, 0.4);
                    }
                }
            }.runTaskLater((Plugin)Main.instance, 1L);
        }
    }
    
    public void npcCreate(final Player p, final boolean actualDeath) {
        boolean setback = false;
        final NPC npc = this.registry.createNPC(EntityType.PLAYER, p.getName());
        final Location l = p.getLocation();
        if (actualDeath && !(boolean)l.getWorld().getGameRuleValue(GameRule.DO_IMMEDIATE_RESPAWN)) {
            l.getWorld().setGameRule(GameRule.DO_IMMEDIATE_RESPAWN, (Object)true);
            setback = true;
        }
        npc.spawn(l);
        npc.teleport(l, PlayerTeleportEvent.TeleportCause.PLUGIN);
        npc.setProtected(false);
        if (!npc.isSpawned()) {
            return;
        }
        if (!this.config.showNameTag) {
            final LivingEntity entity = (LivingEntity)npc.getEntity();
            entity.setCustomNameVisible(false);
            npc.data().setPersistent("nameplate-visible", (Object)false);
        }
        final Vector v = p.getVelocity();
        if (this.config.keepVelocity) {
            npc.getEntity().setVelocity(v);
        }
        final boolean setback2 = setback;
        npc.getEntity().setMetadata("DeathAnimation", (MetadataValue)new FixedMetadataValue((Plugin)Main.instance, (Object)"anything"));
        final boolean shouldSneak = p.isSneaking();
        final boolean shouldSprint = p.isSprinting();
        if (shouldSneak) {
            ((Player)npc.getEntity()).setSneaking(true);
        }
        if (shouldSprint) {
            ((Player)npc.getEntity()).setSprinting(true);
        }
        new BukkitRunnable() {
            public void run() {
                if (PlayerDeath.this.config.keepVelocity) {
                    npc.getEntity().setVelocity(v);
                }
                if (shouldSneak) {
                    ((Player)npc.getEntity()).setSneaking(true);
                }
                if (shouldSprint) {
                    ((Player)npc.getEntity()).setSprinting(true);
                }
                if (setback2) {
                    l.getWorld().setGameRule(GameRule.DO_IMMEDIATE_RESPAWN, (Object)false);
                }
            }
        }.runTask((Plugin)Main.instance);
        new BukkitRunnable() {
            public void run() {
                ((Damageable)npc.getEntity()).setHealth(0.0);
            }
        }.runTaskLater((Plugin)Main.instance, 2L);
        new BukkitRunnable() {
            public void run() {
                npc.destroy();
            }
        }.runTaskLater((Plugin)Main.instance, 20L);
    }
    
    @EventHandler(ignoreCancelled = true, priority = EventPriority.HIGHEST)
    public void onPlayerDamage(final EntityDamageEvent e) {
        if (!this.config.enabled) {
            return;
        }
        if (!(e.getEntity() instanceof Player)) {
            return;
        }
        final Player p = (Player)e.getEntity();
        if (this.cantDamage.contains(p) && e.getCause() != EntityDamageEvent.DamageCause.VOID) {
            if (e instanceof EntityDamageByEntityEvent) {
                final Entity damager = ((EntityDamageByEntityEvent)e).getDamager();
                if (damager instanceof Player) {
                    ((Player)damager).playSound(p.getLocation(), Sound.ENTITY_ITEM_BREAK, 1.0f, 1.0f);
                }
            }
            e.setCancelled(true);
            return;
        }
        if (!this.config.alternateMethod || this.cancellable) {
            return;
        }
        if (e.getDamage() < p.getHealth()) {
            return;
        }
        if (e.isCancelled()) {
            return;
        }
        final PlayerInventory i = p.getInventory();
        if (i.getItemInMainHand() != null && i.getItemInMainHand().getType() == Material.TOTEM_OF_UNDYING) {
            return;
        }
        if (i.getItemInOffHand() != null && i.getItemInOffHand().getType() == Material.TOTEM_OF_UNDYING) {
            return;
        }
        e.setCancelled(true);
        p.setLastDamageCause(e);
        final List<ItemStack> drops = new ArrayList<ItemStack>();
        int expToDrop = 0;
        final boolean keepInventory = (boolean)p.getWorld().getGameRuleValue(GameRule.KEEP_INVENTORY);
        if (!keepInventory) {
            for (final ItemStack item : p.getInventory().getContents()) {
                drops.add(item);
            }
            expToDrop = Math.min(100, XpHandler.getTotalExperience(p));
        }
        final PlayerDeathEvent deathEvent = new PlayerDeathEvent(p, (List)drops, expToDrop, 0, 0, 0, (String)null);
        final PluginManager pm = Bukkit.getPluginManager();
        deathEvent.setKeepInventory(keepInventory);
        deathEvent.setDeathMessage(this.dmh.getDeathMessage(p, e));
        pm.callEvent((Event)deathEvent);
        this.fakeRespawn(p, deathEvent);
    }
    
    public void fakeRespawn(final Player p, final PlayerDeathEvent deathEvent) {
        final String deathMessage = deathEvent.getDeathMessage();
        final int droppedEXP = deathEvent.getDroppedExp();
        boolean dropXP = true;
        final List<ItemStack> drops = deathEvent.getDrops();
        final boolean keepLevel = deathEvent.getKeepLevel();
        final int newEXP = deathEvent.getNewExp();
        final int newLevel = deathEvent.getNewLevel();
        if (this.cancellable) {
            if (deathEvent.shouldPlayDeathSound()) {
                p.playSound(p.getLocation(), deathEvent.getDeathSound(), deathEvent.getDeathSoundCategory(), deathEvent.getDeathSoundVolume(), deathEvent.getDeathSoundPitch());
            }
            dropXP = deathEvent.shouldDropExperience();
        }
        final Location l = p.getLocation();
        if (!deathEvent.getKeepInventory()) {
            p.getInventory().clear();
            for (final ItemStack item : drops) {
                if (item != null) {
                    p.getWorld().dropItemNaturally(l, item);
                }
            }
        }
        if (!keepLevel) {
            XpHandler.setTotalExperience(p, 0);
            p.setTotalExperience(deathEvent.getNewTotalExp());
            p.setLevel(newLevel);
            p.setExp((float)newEXP);
        }
        p.closeInventory();
        for (final Player online : Bukkit.getOnlinePlayers()) {
            online.sendMessage(deathMessage);
        }
        if (dropXP && deathEvent.getDroppedExp() > 0) {
            ((ExperienceOrb)p.getWorld().spawn(p.getLocation(), (Class)ExperienceOrb.class)).setExperience(droppedEXP);
        }
        p.incrementStatistic(Statistic.DEATHS);
        p.setStatistic(Statistic.TIME_SINCE_DEATH, 0);
        if (p.getLastDamageCause() instanceof EntityDamageByEntityEvent) {
            final Entity entity = ((EntityDamageByEntityEvent)p.getLastDamageCause()).getDamager();
            if (entity != null && entity.getType() == EntityType.PLAYER) {
                final Player playerKiller = (Player)entity;
                playerKiller.incrementStatistic(Statistic.PLAYER_KILLS);
            }
        }
        Location spawnLocation = p.getBedSpawnLocation();
        boolean bedSpawn = true;
        if (spawnLocation == null) {
            spawnLocation = Main.instance.getServer().getWorlds().get(0).getSpawnLocation();
            bedSpawn = false;
        }
        final PlayerRespawnEvent respawnEvent = new PlayerRespawnEvent(p, spawnLocation, bedSpawn, false);
        final PluginManager pm = Bukkit.getPluginManager();
        pm.callEvent((Event)respawnEvent);
        final Location spawn = respawnEvent.getRespawnLocation();
        this.npcCreate(p, false);
        try {
            p.setHealth(p.getMaxHealth());
            p.setFireTicks(0);
            p.setFallDistance(0.0f);
            p.setFoodLevel(20);
            p.setSaturation(5.0f);
            p.setExhaustion(0.0f);
            p.setTicksLived(1);
            try {
                p.setArrowsInBody(0);
            }
            catch (final Throwable t) {}
            for (final PotionEffect potionEffect : p.getActivePotionEffects()) {
                p.removePotionEffect(potionEffect.getType());
            }
            p.closeInventory();
            p.setLastDamageCause((EntityDamageEvent)null);
        }
        catch (final Throwable rock) {
            rock.printStackTrace();
        }
        p.getWorld().playSound(l, Sound.ENTITY_PLAYER_DEATH, SoundCategory.PLAYERS, 1.0f, 1.0f);
        p.teleport(spawn);
        p.setVelocity(new Vector(0, 0, 0));
    }
    
    @EventHandler
    public void onPlayerQuit(final PlayerQuitEvent e) {
        if (!this.config.enabled) {
            return;
        }
        if (this.cantDamage.contains(e.getPlayer())) {
            this.cantDamage.remove(e.getPlayer());
        }
    }
    
    @EventHandler
    public void onPlayerDamagePlayer(final EntityDamageByEntityEvent e) {
        if (!this.config.enabled) {
            return;
        }
        if (e.getDamager() instanceof Player) {
            final Player p = (Player)e.getDamager();
            if (this.cantDamage.contains(p)) {
                this.cantDamage.remove(p);
            }
        }
    }
}
